package com.wizarpos.difffingerprint.demo;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.os.RemoteException;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.cloudpos.DeviceException;
import com.cloudpos.OperationResult;
import com.cloudpos.POSTerminal;
import com.cloudpos.TimeConstants;
import com.cloudpos.fingerprint.Fingerprint;
import com.cloudpos.fingerprint.FingerprintDevice;
import com.cloudpos.fingerprint.FingerprintOperationResult;
import com.digitalpersona.uareu.Engine;
import com.digitalpersona.uareu.Fmd;
import com.digitalpersona.uareu.Fmd.Format;
import com.digitalpersona.uareu.Importer;
import com.digitalpersona.uareu.UareUException;
import com.digitalpersona.uareu.UareUGlobal;
import com.upek.android.ptapi.PtConstants;
import com.upek.android.ptapi.PtException;
import com.wizarpos.difffingerprint.mgr.FPMgrImpl;
import com.wizarpos.difffingerprint.mgr.IFPMgr;
import com.wizarpos.difffingerprint.R;
import com.wizarpos.util.Logger;
import com.wizarpos.util.TextViewUtil;

public class MainActivity extends Activity implements View.OnClickListener {
    protected TextView log_text;
    protected Handler mHandler = null;
    private Importer im = null;
    private Engine m_engine = null;
    private FingerprintDevice fingerprintDevice = null;
    private String deviceType = "";

    @SuppressLint("HandlerLeak")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Button loadcross = (Button) this.findViewById(R.id.btn_loading_crossmatch);
        Button loadtuzheng = (Button) this.findViewById(R.id.btn_loading_tuzheng);
        Button enroll = (Button) this.findViewById(R.id.btn_fingerprint_enroll);
        Button compare = (Button) this.findViewById(R.id.btn_fingerprint_compare);
        loadcross.setOnClickListener(this);
        loadtuzheng.setOnClickListener(this);
        enroll.setOnClickListener(this);
        compare.setOnClickListener(this);
        log_text = (TextView) this.findViewById(R.id.text_result);
        log_text.setMovementMethod(ScrollingMovementMethod.getInstance());
        deviceType = checkDeviceType();
        try {
            im = UareUGlobal.GetImporter();
            m_engine = UareUGlobal.GetEngine();
        } catch (Exception e) {
            e.printStackTrace();
        }
        mHandler = new Handler() {
            @Override
            public void handleMessage(Message msg) {
                String str = msg.obj + "\n";
                if (msg.what == R.id.log_default) {
                    log_text.setText(str);
                } else if (msg.what == R.id.log_success) {
                    TextViewUtil.infoBlueTextView(log_text, str);
                } else if (msg.what == R.id.log_failed) {
                    TextViewUtil.infoRedTextView(log_text, str);
                } else if (msg.what == R.id.log_normal) {
                    TextViewUtil.infoGRAYTextView(log_text, str);
                } else if (msg.what == R.id.log_clean) {
                    log_text.setText("");
                }
            }
        };
    }

    private void sendLog(String log, int what) {
        Message msg = new Message();
        msg.what = what;
        msg.obj = log;
        mHandler.sendMessage(msg);
    }

    private byte[] fingerpringtBs;
    private byte[] enrollBs;

    @Override
    public void onClick(View view) {
        int index = view.getId();
        switch (index) {
            case R.id.btn_loading_crossmatch:
                fingerpringtBs = readFingerprintData(1);
                break;
            case R.id.btn_loading_tuzheng:
                fingerpringtBs = readFingerprintData(2);
                break;
            case R.id.btn_fingerprint_enroll:
                sendLog("this device type is:" + deviceType, R.id.log_normal);
                if (deviceType.contains("crossmatch")) {
                    crossmatchEnroll();
                } else if (deviceType.contains("tuzheng")) {
                    tuzhengEnroll();
                }
                break;
            case R.id.btn_fingerprint_compare:
                if (deviceType.contains("crossmatch")) {
                    crossmatchCompare(fingerpringtBs, enrollBs);
                } else if (deviceType.contains("tuzheng")) {
                    tuzhengCompare(fingerpringtBs, enrollBs);
                } else {
                    sendLog("deviceType not support.", R.id.log_failed);
                }
                break;
        }
    }

    private String checkDeviceType() {
        return getProperty("wp.fingerprint.model", "");
    }

    public byte[] readFingerprintData(int fType) {
        FileInputStream fis = null;
        try {
            String ftypeStr = "";
            if (fType == 1) {
                ftypeStr = "fingerprint_data_crossmatch";
            } else if (fType == 2) {
                ftypeStr = "fingerprint_data_tuzhengbig";
            }
            File file = new File(Environment.getExternalStorageDirectory() + "/Download", ftypeStr);
            fis = new FileInputStream(file);
            byte[] buffer = new byte[fis.available()];
            fis.read(buffer);
            fis.close();
            sendLog("load " + ftypeStr + "," + buffer.length, R.id.log_success);
            Logger.debug("read:" + HexString.bufferToHex(buffer));
            return buffer;
        } catch (Exception ex) {
            ex.printStackTrace();
            sendLog("not find fingerprint data file." + ex.getMessage(), R.id.log_failed);
            try {
                if (fis != null) {
                    fis.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    private Thread fingerTh = null;

    public void crossmatchEnroll() {
        if (fingerTh == null || fingerTh.getState() == Thread.State.TERMINATED) {
            fingerTh = new Thread() {
                @SuppressWarnings("unused")
                public void run() {
                    IFPMgr fpMgr = FPMgrImpl.getInstance();
                    try {
                        fpMgr.open(MainActivity.this);
                        fpMgr.deleteAll(MainActivity.this);
                        sendLog("press finger keep,enroll ", R.id.log_success);
                        try {
                            byte[] image = fpMgr.GrabImage(PtConstants.PT_GRAB_TYPE_508_508_8_SCAN508_508_8);
                            int iWidth = fpMgr.getImagewidth();
                            Fmd fm = ConvertImgToIsoTemplate(image, iWidth);
                            if (fm != null) {
                                byte[] bs = fm.getData();
                                enrollBs = bs;
                                writeBytesToFile(bs);
                                Log.e("crossmatchEnroll fp", "" + HexString.bufferToHex(bs));
                                sendLog("crossmatch enroll success!", R.id.log_success);
                            } else {
                                Log.e("fm", "fm is null");
                                sendLog("crossmatch enroll failed!", R.id.log_failed);
                            }
                        } catch (PtException e) {
                            sendLog("exception " + e.getMessage(), R.id.log_success);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        sendLog("exception occured !" + e.getMessage(), R.id.log_failed);
                    } finally {
                        fpMgr.close();
                    }
                }
            };
            fingerTh.start();
        }
    }

    private void tuzhengEnroll() {
        if (fingerTh == null || fingerTh.getState() == Thread.State.TERMINATED) {
            fingerTh = new Thread() {
                @SuppressWarnings("unused")
                public void run() {
                    openDevice();
                    sendLog("press finger keep,enroll ", R.id.log_success);
                    try {
                        FingerprintOperationResult operationResult = fingerprintDevice.waitForFingerprint(TimeConstants.FOREVER);
                        if (operationResult.getResultCode() == OperationResult.SUCCESS) {
                            Fingerprint finger = operationResult.getFingerprint(0, 0);
                            byte[] bs = finger.getFeature();
                            enrollBs = bs;
                            Log.e("tuzhengEnroll fp", "" + HexString.bufferToHex(bs));
                            writeBytesToFile(bs);
                            sendLog("tuzhengEnroll enroll success!", R.id.log_success);
                        } else {
                            sendLog("get TUZHENG fingerprint failed,please try again", R.id.log_failed);
                        }

                    } catch (Exception e) {
                        e.printStackTrace();
                        sendLog("get TUZHENG fingerprint failed,please try again", R.id.log_failed);
                    } finally {
                        closeDevice();
                    }
                }
            };
            fingerTh.start();
        }
    }

    private void openDevice() {
        //need permission,view AndroidManifest.xml
        try {
            fingerprintDevice = (FingerprintDevice) POSTerminal.getInstance(MainActivity.this).getDevice("cloudpos.device.fingerprint");
            fingerprintDevice.open(2);
        } catch (DeviceException e) {
            e.printStackTrace();
        }
    }

    private void closeDevice() {
        try {
            fingerprintDevice.close();
        } catch (DeviceException e) {
            e.printStackTrace();
        }
    }

    private Fmd ConvertImgToIsoTemplate(byte[] aImage, int iWidth) {
        if (aImage == null) {
            return null;
        }
        int iHeight = aImage.length / iWidth;
        try {
            Engine engine = UareUGlobal.GetEngine();
            Fmd fmd = engine.CreateFmd(aImage, iWidth, iHeight, 500, 0, 0, Format.ANSI_378_2004);
            Log.i("BasicSample", "Import a Fmd from a raw image OK");
            return fmd;
        } catch (UareUException e) {
            Log.d("BasicSample", "Import Raw Image Fail", e);
            return null;
        }
    }

    private void crossmatchCompare(byte[] bsF1, byte[] bsF2) {
        if (bsF1 == null || bsF1.length == 0) {
            sendLog("not loading data", R.id.log_failed);
            return;
        }
        if (bsF2 == null || bsF2.length == 0) {
            sendLog("no enroll fingerprint", R.id.log_failed);
            return;
        }
        int score = -655355; //compare myself, score: 0=match
        try {
            Fmd fm1 = im.ImportFmd(bsF1, Format.ANSI_378_2004, Format.ANSI_378_2004);
            Fmd fm2 = im.ImportFmd(bsF2, Format.ANSI_378_2004, Format.ANSI_378_2004);
            score = m_engine.Compare(fm1, 0, fm2, 0);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (score == 0) {
            sendLog("crossmatch Compare success, score=" + score, R.id.log_success);
        } else {
            sendLog("crossmatch Compare failed, score=" + score, R.id.log_failed);
        }
        Log.i("crossmatchCompare", String.format("score= %d", score));
    }

    private void tuzhengCompare(final byte[] bsF1, final byte[] bsF2) {
        if (bsF1 == null || bsF1.length == 0) {
            sendLog("not loading data", R.id.log_failed);
            return;
        }
        if (bsF2 == null || bsF2.length == 0) {
            sendLog("no enroll fingerprint", R.id.log_failed);
            return;
        }
        sendLog("tuzhengCompare fingerprint,wait...", R.id.log_failed);

        new Thread() {
            @Override
            public void run() {
                super.run();
                try {
                    openDevice();
                    /**
                     * Compares two fingerprint datas by format.
                     * @param bsF1 : the byte buffer of the first Fingerprint Minutiae Data
                     * @param format1 : the arryBuffer1's format
                     * 						0:ISO IEC 19794-2-2005 Fingerprint Minutiae Data format
                     *						1:ANSI INSITS 378-2004 Fingerprint Minutiae Data format
                     * @param bsF2 : the byte buffer of the second Fingerprint Minutiae Data
                     * @param format1 : the arryBuffer2's format
                     * 						0:ISO IEC 19794-2-2005 Fingerprint Minutiae Data format
                     *						1:ANSI INSITS 378-2004 Fingerprint Minutiae Data format
                     * @return Dissimilarity score.
                     * */
                    int matchResult = fingerprintDevice.compareByFormat(bsF1, 1, bsF2, 1);
                    if (matchResult == 0) {
                        sendLog("tuzheng Compare success, matchResult=" + matchResult, R.id.log_success);
                    } else {
                        sendLog("tuzheng Compare failed, matchResult=" + matchResult, R.id.log_failed);
                    }
                } catch (DeviceException e) {
                    sendLog("tuzheng Compare failed DeviceException", R.id.log_failed);
                    e.printStackTrace();
                } catch (RemoteException e) {
                    e.printStackTrace();
                } finally {
                    closeDevice();
                }
            }
        }.start();
    }

    public static String getProperty(String key, String defaultValue) {
        String value = defaultValue;
        try {
            Class<?> c = Class.forName("android.os.SystemProperties");
            Method get = c.getMethod("get", String.class, String.class);
            value = (String) (get.invoke(c, key, defaultValue));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return value;
    }


    public void writeBytesToFile(byte[] bs) throws IOException {
        String pathName = Environment.getExternalStorageDirectory() + "/Download/" + "fingerprint_" + deviceType;
        OutputStream out = new FileOutputStream(pathName);
        InputStream is = new ByteArrayInputStream(bs);
        byte[] buff = new byte[1024];
        int len = 0;
        while ((len = is.read(buff)) != -1) {
            out.write(buff, 0, len);
        }
        is.close();
        out.close();
        sendLog("fingerprint,save path=" + pathName, R.id.log_success);
    }
}
